@@ -1,6 +1,6 @@
 # vim: set fileencoding=utf-8 :
 # GNU Solfege - free ear training software
-# Copyright (C) 2009, 2010  Tom Cato Amundsen
+# Copyright (C) 2009, 2010, 2011  Tom Cato Amundsen
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,16 +17,24 @@
 
 from __future__ import absolute_import
 
+import locale
 import logging
+import sys
 
+import gobject
 import gtk
 
 import solfege
-from solfege import cfg
 from solfege import frontpage
 from solfege import gu
 from solfege import lessonfile
 
+def decode_entry_string(s):
+    if sys.platform == 'win32':
+        return s.decode("mbcs", "replace")
+    return s.decode(locale.getpreferredencoding(), "decode")
+
+
 class SelectWinBase(gtk.ScrolledWindow):
     """
     Base class for the classes that display the front page and the
@@ -122,7 +130,7 @@ class SelectWinBase(gtk.ScrolledWindow):
         # ubuntu 10.04 and +4 on MS Windows XP.
         h = self.g_box.size_request()[1] + 4
         if h > gtk.gdk.screen_height() * 0.8:
-            h = gtk.gdk.screen_height() *0.8
+            h = int(gtk.gdk.screen_height() *0.8)
         self.set_size_request(w, h if h > self.m_min_height else self.m_min_height)
         # Then we check if we have to move the app window higher up on the
         # screen so that the whole window is visible.
@@ -165,8 +173,13 @@ class ExerciseView(SelectWinBase):
         do_sg = True
         if len(page[0]) > 0 and isinstance(page[0][0], frontpage.LinkList):
             count = 0
-            for p in page[0]:
-                count += len(page[0][0])
+            for column in page:
+                assert isinstance(column, frontpage.Column)
+                for linklist in column:
+                    assert isinstance(linklist, (frontpage.LinkList,
+                                                 frontpage.Paragraph))
+                    if isinstance(linklist, frontpage.LinkList):
+                        count += len(linklist)
             if count > 80:
                 do_sg = False
         for column in page:
@@ -179,6 +192,7 @@ class ExerciseView(SelectWinBase):
             for sect_id, linklist in enumerate(column):
                 if isinstance(linklist, frontpage.Paragraph):
                     label = gtk.Label(linklist.m_text)
+                    label.set_use_markup(True)
                     label.set_line_wrap(True)
                     label.set_alignment(0.0, 0.5)
                     vbox.pack_start(label, False)
@@ -258,7 +272,7 @@ class ExerciseView(SelectWinBase):
                                 box.pack_start(gtk.Label(_("passed, %.1f%%") % (result * 100)))
                             if passed == False:
                                 box.pack_start(gtk.Label(_("failed, %.1f%%") % (result * 100)))
-                            # do nothing if passed == None since 
+                            # do nothing if passed == None since
                         linkbox.pack_start(box, True)
                     else:
                         linkbox.pack_start(label, True)
@@ -268,21 +282,23 @@ class ExerciseView(SelectWinBase):
                 label.set_data('last', True)
         self.g_hbox.show_all()
         self.adjust_scrolledwin_size()
-        self.get_vadjustment().value = 0.0
+        self.get_vadjustment().set_value(0.0)
     def on_page_link_clicked(self, btn, link):
         self.g_searchbox.hide()
+        self.emit('link-clicked', self.m_page)
         self.display_data(link)
     def on_link_clicked(self, w, filename):
+        self.emit('link-clicked', self.m_page)
         solfege.app.practise_lessonfile(filename)
     def display_search_result(self, searchfor, result, result_C, display_only_tests=False):
         self._display_data(
           frontpage.Page(u'',
             frontpage.Column([
               frontpage.LinkList(
-            _("Search results for “%s”:" % self.g_searchentry.get_text()),
+            _(u"Search results for “%s”:") % self.g_searchentry.get_text(),
               result),
               frontpage.LinkList(
-            _("C-locale search results for “%s”:" % self.g_searchentry.get_text()),
+            _(u"C-locale search results for “%s”:") % self.g_searchentry.get_text(),
               result_C),
               ])))
     def on_end_practise(self, w=None):
@@ -294,7 +310,7 @@ class ExerciseView(SelectWinBase):
         only_tests - True if we should only return exercises that have
                      tests
         """
-        logging.debug("search: '%s'" % substring)
+        logging.debug("search: '%s'", substring)
         match_func = {
             False: lambda filename: substring in _no_xgettext(lessonfile.infocache.get(filename, 'title')).lower(),
             True: lambda filename: substring in lessonfile.infocache.get(filename, 'title').lower()
@@ -306,7 +322,7 @@ class ExerciseView(SelectWinBase):
         page = frontpage.Page(listitems=frontpage.Column())
         cur_topic = None
         # the last topic appended to the page
-        last_topic = None 
+        last_topic = None
         found = set()
         for child in self.m_page.iterate_flattened():
             if isinstance(child, frontpage.LinkList):
@@ -324,6 +340,11 @@ class ExerciseView(SelectWinBase):
                     pass
         return page
 
+gobject.signal_new('link-clicked', ExerciseView,
+                   gobject.SIGNAL_RUN_FIRST,
+                   gobject.TYPE_NONE,
+                   (gobject.TYPE_PYOBJECT,))
+
 
 class FrontPage(ExerciseView):
     def display_data(self, data, display_only_tests=False,
@@ -332,8 +353,8 @@ class FrontPage(ExerciseView):
                            show_topics)
         self.g_searchentry.grab_focus()
     def on_search(self, *button):
-        search_for = self.g_searchentry.get_text().lower()
-        logging.debug("FrontPage.on_search '%s'" % search_for)
+        search_for = decode_entry_string(self.g_searchentry.get_text().lower())
+        logging.debug(u"FrontPage.on_search '%s'", search_for)
         page = self._search(search_for, False, False)
         page_C = self._search(search_for, True, False)
         page_C.m_name = _('Search untranslated lesson titles')
@@ -345,12 +366,13 @@ class FrontPage(ExerciseView):
 
 class TestsView(ExerciseView):
     def on_link_clicked(self, w, filename):
+        self.emit('link-clicked', self.m_page)
         solfege.app.test_lessonfile(filename)
     def display_data(self, data=None, is_search_result=False, show_topics=False):
         self._display_data(data, True, is_search_result, show_topics)
         self.g_searchentry.grab_focus()
     def on_search(self, *button):
-        search_for = self.g_searchentry.get_text().lower()
+        search_for = decode_entry_string(self.g_searchentry.get_text().lower())
         page = self._search(search_for, False, True)
         page_C = self._search(search_for, True, True)
         page_C.m_name = u'Search exercises without translating them'
@@ -363,7 +385,7 @@ class TestsView(ExerciseView):
 class SearchView(ExerciseView):
     def __init__(self, infotext, fields=('link',)):
         ExerciseView.__init__(self, fields)
-        page = """FileHeader(1, 
+        page = """FileHeader(1,
         Page(u'', [
          Column([
           Paragraph('%s'),
@@ -377,9 +399,9 @@ class SearchView(ExerciseView):
         self._display_data(data)
         self.g_searchentry.grab_focus()
     def on_search(self, *button):
-        search_for = self.g_searchentry.get_text().lower()
+        search_for = decode_entry_string(self.g_searchentry.get_text().lower())
         lessonfile.infocache.update_modified_files()
-        self.display_search_result(self.g_searchentry.get_text(), [
+        self.display_search_result(decode_entry_string(self.g_searchentry.get_text()), [
             filename for filename in lessonfile.infocache._data
             if search_for in _(lessonfile.infocache.get(filename, 'title')).lower()
         ], [
