@@ -1,5 +1,5 @@
 # GNU Solfege - free ear training software
-# Copyright (C) 2010  Tom Cato Amundsen
+# Copyright (C) 2010, 2011  Tom Cato Amundsen
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,23 +16,15 @@
 
 from __future__ import absolute_import
 
-import random
-
-import gtk
-
 from solfege import gu
 from solfege import abstract
-from solfege import gu
 from solfege import lessonfile
 from solfege import mpd
 from solfege import soundcard
-from solfege import utils
 
-from solfege.mpd.parser import Lexer
-from solfege.mpd.duration import Duration
-from solfege.mpd.rat import Rat
+from solfege.mpd import Duration
 from solfege.mpd import elems
-from solfege.rhythmwidget import RhythmWidget, Controller
+from solfege.mpd import RhythmWidget, RhythmWidgetController
 
 
 class Teacher(abstract.Teacher):
@@ -81,7 +73,7 @@ class Gui(abstract.LessonbasedGui):
         abstract.LessonbasedGui.__init__(self, teacher)
         self.g_w = RhythmWidget()
         self.practise_box.pack_start(self.g_w, False)
-        self.g_c = Controller(self.g_w)
+        self.g_c = RhythmWidgetController(self.g_w)
         self.practise_box.pack_start(self.g_c, False)
         self.g_flashbar = gu.FlashBar()
         self.practise_box.pack_start(self.g_flashbar, False)
@@ -128,12 +120,15 @@ class Gui(abstract.LessonbasedGui):
         self.g_w.grab_focus()
         self.m_t.play_question()
     def give_up(self, *w):
-        self.m_t.m_P.m_question_score.m_staffs.append(self.g_w.m_score.m_staffs[0])
-        self.m_t.m_P.m_question_score.m_staffs[-1].set_parent(self.m_t.m_P.m_question_score)
-        self.m_t.m_P.m_question_score.m_staffs[0].m_label = _("The music played:")
-        self.m_t.m_P.m_question_score.m_staffs[-1].m_label = _("The rhythm you entered:")
-        self.m_t.m_P.m_question_score.create_shortcuts()
-        self.g_w.set_score(self.m_t.m_P.m_question_score, cursor=None)
+        # Make a copy of the question asked. Then attach the staff
+        # the user entered below, set some labels and display it.
+        score_copy = self.m_t.m_P.m_question_score.copy()
+        score_copy.m_staffs.append(self.g_w.m_score.m_staffs[0])
+        score_copy.m_staffs[-1].set_parent(self.m_t.m_P.m_question_score)
+        score_copy.m_staffs[0].m_label = _("The music played:")
+        score_copy.m_staffs[-1].m_label = _("The rhythm you entered:")
+        score_copy.create_shortcuts()
+        self.g_w.set_score(score_copy, cursor=None)
         self.m_t.give_up()
         self.g_c.set_editable(False)
         self.std_buttons_give_up()
