@@ -12,8 +12,8 @@ def do_file(filename):
     m = r.search(s)
     def func(r):
         years = r.group('years')
-        if '2008' not in years:
-            years ="%s, 2008" % years
+        if '2011' not in years:
+            years ="%s, 2011" % years
         return r.group('left') + years + r.group('right')
     s2 = r.sub(func, s)
     if s2 != s:
@@ -33,7 +33,12 @@ def motto(filename):
 
 
 for root, dirs, files in os.walk("."):
+    if root in ('./mylint-tmpdir', './build-branch', './manual-po-tmp'):
+        continue
+    if root.startswith("./backup.bzr.~1~") or root.startswith("./.bzr"):
+        continue
     for f in files:
-        if f.endswith(".py") or f == 'Makefile':
-            do_file(os.path.join(root, f))
-            #motto(os.path.join(root, f))
+        if ((f.endswith(".py") or f == 'Makefile')
+            or root == './exercises/standard/lesson-files'):
+                    do_file(os.path.join(root, f))
+                    motto(os.path.join(root, f))
